#!/usr/bin/perl -Tw
# Copyright 2002, Anders Nordby <anders@fix.no>
#
# This is the web frontend.
# Part of "web interface for approving Majordomo messages".
#
# THIS SOFTWARE IS PROVIDED BY ANDERS NORDBY ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL ANDERS NORDBY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Casual users should only change this line:
my $ini = "/home/anders/bounceweb/bouncewebfrontend.ini";

use Config::IniFiles;
use MIME::Parser;
use Mail::Internet;
use MIME::Words qw(decode_mimewords);

my $cfg = new Config::IniFiles( -file => "$ini");
my $dbhost=$cfg->val( 'bounceweb', 'dbhost' );
my $dbuser=$cfg->val( 'bounceweb', 'dbuser' );
my $dbpw=$cfg->val( 'bounceweb', 'dbpw' );
my $dbname=$cfg->val( 'bounceweb', 'dbname' );
my $dbport=$cfg->val( 'bounceweb', 'dbport' );
my $mychars=$cfg->val( 'bounceweb', 'mychars' );
my $mychars_numbers=$cfg->val( 'bounceweb', 'mychars_numbers' );
$parser = new MIME::Parser;
#$parser->output_dir($cfg->val( 'bounceweb', 'mimetmp' ));
$parser->output_to_core(1);
$limitmsgs=$cfg->val( 'bounceweb', 'limitmsgs' );
$approveprog=$cfg->val( 'bounceweb', 'approveprog' );
$approveconf=$cfg->val( 'bounceweb', 'approveconf' );

use DBI;
#use strict;

sub validchars
{
	# 0: string 1: valid characters
	my $streng = $_[0];

	my $ok = 1;
	my $i = 0;
	while ($ok && $i < length($_[0])) {
		if (index($_[1], substr($_[0],$i,1)) == -1) {
			$ok = 0;
		}
		$i++;
	}
	return($ok);
}

sub cleanup {
	$dbh->disconnect;
}

sub perror {
	print "<p><font color=\"#ff0000\">ERROR: $_[0]</font></p>\n";
	print "</body></html>\n";
	cleanup;
	exit 1;
}

sub listmsg {
	print "<p><h2>Approving of bounced messages for user: $remoteuser</h2></p>";
	$i = 0;
	for $list (@lists) {

		my $dbq_bmsg = $dbh->prepare("select * from bouncemsg where listid=\"$listids[$i]\" and deleted=0 order by msgno desc limit $limitmsgs;");
		$dbq_bmsg->execute;
		my $numrows = $dbq_bmsg->rows;
		print "<hr>\n";	
		if ($numrows > 0) {
			print "<p>Mails for list: $list</p>\n";

			print "<p><table border=1>\n";
			print "<tr><th>Bounce QN<th>Subject<th>From<th>Date<th>Approved by<th>Approved date<th>Delete message<th>Approve message<tr>\n";

			while (my $row = $dbq_bmsg->fetchrow_hashref()) {
				my @msg = split /\n/, $row->{message};
				my $xmail = new Mail::Internet \@msg;
				my $xmsgstr = substr(join("\n", @{$xmail->body}), 0, 1600);
				my $xmsg = $parser->parse_data($xmsgstr);
				my $subj = $xmsg->head->get('Subject');
				if ($subj) {
					chomp($subj);
					$subj = decode_mimewords($subj);
				} else {
					next;
				}
				my $from = $xmsg->head->get('From');
				if ($from) {
					chomp($from);
					$from = decode_mimewords($from);
					$from =~ s@\<@\&lt\;@g;
					$from =~ s@\>@\&gt\;@g;
				} else {
					next;
				}
				my $date = $xmsg->head->get('Date');
				if ($date) {
					chomp($date);
					$date = decode_mimewords($date);
				} else {
					next;
				}

				$approvedate = $row->{approved};
				$approvedbyuid = $row->{approvedbyuid};

				if (!$approvedate) { $approvedate = "Never"; };
				if (($approvedbyuid) && (exists $useridmap{$approvedbyuid})) {
					$approvedby = $useridmap{$approvedbyuid};
				} else {
					$approvedby = "Nobody";
				}
# limit which messages to show here, or earlier:
				print "<tr><td><a href=\"/cgi-bin/bouncemails.pl?showraw\&$row->{msgno}\">$row->{msgno}</a><td>$subj<td>$from<td>$date<td>$approvedby<td>$approvedate<td><a href=\"/cgi-bin/bouncemails.pl?delmsg\&$row->{msgno}\">Delete</a><td><a href=\"/cgi-bin/bouncemails.pl?approvemsg&$row->{msgno}\">Approve</a></tr>\n";
			}
		} else {
			print "<p>No mails for list $list.</p>\n";
		}
		$dbq_bmsg->finish;

		print "</table></p>\n";
		$i++;
	}
}

sub pback {
	my $url = "/cgi-bin/bouncemails.pl?list";
	my $rtime = $_[0];
	if ($rtime) {
		print "<meta http-equiv=\"Refresh\" content=\"$rtime; url=$url\">\n";
	}
	print "<p>Click <a href=\"$url\">here</a> to go back to the list of mails.</p>\n";
}

sub domsg {
	my $mode = $_[0];
	my $mnumber = $_[1];

	my $dbq_bmsg = $dbh->prepare("select listid, message from bouncemsg where msgno=\"$mnumber\" and deleted=0;");
	$dbq_bmsg->execute;
	my $row = $dbq_bmsg->fetchrow_hashref();
	if (grep (/^$row->{listid}$/, @listids)) {
		for ($mode) {
			/^showraw$/	and do {
				print "<p><h2>Showing raw copy of message $mnumber</h2></p>\n";
				pback;
				print "<hr>\n";
				print "<p><xmp>\n";
				my $rawmsg = decode_mimewords ( $row->{message} );
				print "$rawmsg";
				print "</xmp></p>\n";

				last;
			};
			/^delmsg$/	and do {
				print "<p><h2>Deleting message $mnumber</h2></p><hr>\n";
				my $dbq_delmsg = $dbh -> prepare("update bouncemsg set deleted=1 where msgno=\"$mnumber\";");
				$dbq_delmsg->execute;
				$dbq_delmsg->finish;
				print "<p>OK, done.</p>\n";

				pback(4);
				last;
			};
			/^approvemsg$/	and do {
				print "<p><h2>Approving message $mnumber</h2></p><hr>\n";

				if (open(APPROVE, "| $approveprog -f $approveconf")) {
					print APPROVE $row->{message};
					close APPROVE;
				} else {
					perror("Could not open approve program for approving the mail. Notify administrator.");
				}

				my $dbq_approvemsg = $dbh -> prepare("update bouncemsg set approved=now() where msgno=\"$mnumber\";");
				$dbq_approvemsg->execute;
				$dbq_approvemsg->finish;
				$dbq_approvemsg = $dbh -> prepare("update bouncemsg set approvecount=approvecount+1 where msgno=\"$mnumber\";");
				$dbq_approvemsg->execute;
				$dbq_approvemsg->finish;
				$dbq_approvemsg = $dbh -> prepare("update bouncemsg set approvedbyuid=$userid where msgno=\"$mnumber\";");
				$dbq_approvemsg->execute;
				$dbq_approvemsg->finish;

				print "<p>OK, done.</p>\n";
				pback(4);
				last;
			};
		}
	} else {
		print "<p>You do not have access to this message.</p>\n";
	}
	$dbq_bmsg->finish;
}

sub approvemsg {
	my $mnumber = $_[0];
}

$ENV{'PATH'} = "/bin:/sbin:/usr/bin:/usr/sbin";

print "Content-type: text/html\n\n";

$remoteuser=$ENV{'REMOTE_USER'};
#$remoteuser="anders";
if (!$remoteuser) {
	perror("Could not associate user with database.");
} elsif (!validchars($remoteuser, $mychars)) {
	perror("Remote username contains funny characters.");
}

$dbh = DBI->connect("DBI:mysql:database=$dbname:host=$dbhost:port=$dbport",$dbuser,$dbpw,{PrintError=>0});

if (!$dbh) { perror("Could not connect to database $dbname."); }

%usermap = ();
%useridmap = ();
my $dbq_userid = $dbh->prepare("select username, userid from users;");
$dbq_userid->execute;
while (my $row = $dbq_userid->fetchrow_hashref()) {
	$usermap{$row->{username}} = $row->{userid};
	$useridmap{$row->{userid}} = $row->{username};
}
$userid = $usermap{$remoteuser};

if (!$userid) { perror("Could not find userid."); }
chomp($userid);

my $dbq_acl = $dbh->prepare("select lists.listname, lists.listid from lists, acl where lists.listid=acl.listid and acl.userid=\"$userid\";");
$dbq_acl->execute;
my $i = 0;
while (my $row = $dbq_acl->fetchrow_hashref()) {
	$lists[$i] = $row->{listname};
	$listids[$i] = $row->{listid};
	$i++;
}
$dbq_acl->finish;

print "<html><body bgcolor=\"#ffffff\" text=\"#000000\" link=\"#aaaaaa\" vlink=\"#\">\n";

@ARGV = ("") unless @ARGV;
@ARGS = split /\\&/, $ARGV[0];
for ($ARGS[0]) {
	/^list$/		and do {
		listmsg;
		last;
	};
	/^(showraw|approvemsg|delmsg)$/	and do {
		if ($ARGS[1]) {
			if (validchars($ARGS[1], $mychars_numbers)) {
				domsg($ARGS[0], $ARGS[1]);
			} else {
				perror("Invalid characters in message number.");
			}
		} else {
				perror("No message number entered.");
		}
		last;
	};
	perror("Invalid mode.");
}

print "<hr>\n";
print "</body></html>\n";

cleanup;
